###################################################
### chunk number 1: set-opts
###################################################
  glop <- options(keep.source=TRUE,width=60,continue=" ",prompt=" ")
  library(pomp)
  pdf.options(useDingbats=FALSE)
  set.seed(5384959)


###################################################
### chunk number 2:  eval=FALSE
###################################################
## require(pomp)
## data(ou2)
## show(ou2)


###################################################
### chunk number 3: ou2-rprocess
###################################################
  ou2.rprocess <- function (xstart, times, params, paramnames, ...) {
    nvar <- nrow(xstart)
    npar <- nrow(params)
    nrep <- ncol(xstart)
    ntimes <- length(times)
    ## get indices of the various parameters in the 'params' matrix
    ## C uses zero-based indexing!
    parindex <- match(paramnames,rownames(params))-1
    array(
	  .C("_ou2_adv",
	     X = double(nvar*nrep*ntimes),
	     xstart = as.double(xstart),
	     par = as.double(params),
	     times = as.double(times),
	     n = as.integer(c(nvar,npar,nrep,ntimes)),
	     parindex = as.integer(parindex),
	     DUP = FALSE,
	     NAOK = TRUE,
	     PACKAGE = "pomp"
	     )$X,
	  dim=c(nvar,nrep,ntimes),
	  dimnames=list(rownames(xstart),NULL,NULL)
	  )
  }


###################################################
### chunk number 4: ou2-dprocess
###################################################
  ou2.dprocess <- function (x, times, params, log, paramnames, ...) {
    nvar <- nrow(x)
    npar <- nrow(params)
    nrep <- ncol(x)
    ntimes <- length(times)
    parindex <- match(paramnames,rownames(params))-1
    array(
	  .C("_ou2_pdf",
             d = double(nrep*(ntimes-1)),
             X = as.double(x),
	     par = as.double(params),
	     times = as.double(times),
	     n = as.integer(c(nvar,npar,nrep,ntimes)),
	     parindex = as.integer(parindex),
             give_log=as.integer(log),
	     DUP = FALSE,
	     NAOK = TRUE,
	     PACKAGE = "pomp"
	     )$d,
	  dim=c(nrep,ntimes-1)
	  )
  }


###################################################
### chunk number 5: 
###################################################
ou2 <- pomp( 
	    data=data.frame(
              time=seq(1,100),
	      y1=NA,
	      y2=NA
	      ),
	    times="time",
	    t0=0,
	    rprocess = ou2.rprocess,
	    dprocess = ou2.dprocess,
	    dmeasure = "_ou2_normal_dmeasure",
	    rmeasure = "_ou2_normal_rmeasure",
            paramnames=c(
              "alpha.1","alpha.2","alpha.3","alpha.4",
              "sigma.1","sigma.2","sigma.3",
              "tau"
              ),
            statenames = c("x1","x2"),
            obsnames = c("y1","y2"),
            PACKAGE="pomp"
	    )


###################################################
### chunk number 6: view-ou2-source eval=FALSE
###################################################
## edit(file=system.file("examples/ou2.c",package="pomp"))


###################################################
### chunk number 7: 
###################################################
theta <- c(
           alpha.1=0.8, alpha.2=-0.5, alpha.3=0.3, alpha.4=0.9,
           sigma.1=3, sigma.2=-0.5, sigma.3=2,
           tau=1, 
           x1.0=-3, x2.0=4
           )


###################################################
### chunk number 8: 
###################################################
tic <- Sys.time()
x <- simulate(ou2,params=theta,nsim=500,seed=80073088L)
toc <- Sys.time()
print(toc-tic)


###################################################
### chunk number 9: sir-def
###################################################
pomp(
     data=data.frame(
       time=seq(from=1/52,to=4,by=1/52),
       reports=NA
       ),
     times="time",
     t0=0,
     ## native routine for the process simulator:
     rprocess=euler.sim(
       step.fun="_sir_euler_simulator", 
       delta.t=1/52/20
       ),
     ## native routine for the skeleton:
     skeleton.vectorfield="_sir_ODE", 
     ## binomial measurement model:
     rmeasure="_sir_binom_rmeasure", 
     dmeasure="_sir_binom_dmeasure", 
     ## name of the shared-object library containing the native routines:
     PACKAGE="pomp", 
     ## the order of the observable assumed in the native routines:
     obsnames=c("reports"),
     ## the order of the state variables assumed in the native routines:
     statenames=c("S","I","R","cases","W"),
     ## the order of the parameters assumed in the native routines:
     paramnames=c(
       "gamma","mu","iota","beta1","beta.sd",
       "pop","rho","nbasis","degree","period"
       ), 
     ## reset cases to zero at each new observation:
     zeronames=c("cases"),      
     initializer=function(params,t0,...){
       p <- exp(params)
       with(
            as.list(p),
            {
              fracs <- c(S.0,I.0,R.0)
              x0 <- round(c(pop*fracs/sum(fracs),0,0))
              names(x0) <- c("S","I","R","cases","W")
              x0
            }
            )
     }
     ) -> sir


###################################################
### chunk number 10: view-sir-source eval=FALSE
###################################################
## edit(file=system.file("examples/sir.c",package="pomp"))


###################################################
### chunk number 11: sir-sim
###################################################
params <- c(
            gamma=26,mu=0.02,iota=0.01,
            beta1=1200,beta2=1800,beta3=600,
            beta.sd=1e-3,                      
            pop=2.1e6,
            rho=0.6,
            S.0=26/1200,I.0=0.001,R.0=1-0.001-26/1200
            )                                                     

sir <- simulate(sir,params=c(log(params),nbasis=3,degree=3,period=1),seed=3493885L)

tic <- Sys.time()
sims <- simulate(sir,nsim=10)
toc <- Sys.time()
print(toc-tic)

tic <- Sys.time()
traj <- trajectory(sir,hmax=1/52)
toc <- Sys.time()
print(toc-tic)


###################################################
### chunk number 12: 
###################################################
data(ou2)
true.p <- coef(ou2)
x0 <- init.state(ou2)
x0
new.p <- cbind(true.p,true.p,true.p)
new.p["x1.0",] <- 1:3
init.state(ou2,params=new.p)


###################################################
### chunk number 13: 
###################################################
x <- rprocess(ou2,xstart=x0,times=time(ou2,t0=T),params=as.matrix(true.p))
dim(x)
x[,,1:5]


###################################################
### chunk number 14: 
###################################################
x <- x[,,-1,drop=F]
y <- rmeasure(ou2,x=x,times=time(ou2),params=as.matrix(true.p))
dim(y)
y[,,1:5]


###################################################
### chunk number 15: 
###################################################
fp <- dprocess(ou2,x=x,times=time(ou2),params=as.matrix(true.p))
dim(fp)
fp[,36:40]


###################################################
### chunk number 16: 
###################################################
fm <- dmeasure(ou2,y=y[,1,],x=x,times=time(ou2),params=as.matrix(true.p))
dim(fm)
fm[,36:40]


###################################################
### chunk number 17: all-data-loaable eval=FALSE
###################################################
## data(package="pomp")


###################################################
### chunk number 18: restore-opts
###################################################
options(glop)


